# ============================================================
# Система: Единая библиотека, Центр ИИ НИУ ВШЭ
# Модуль: MLComponent
# Авторы: Полежаев В.А., Хританков А.С.
# Дата создания: 2024 г.
# ============================================================
from typing import List, Protocol, Literal, Optional, Tuple, Union, Any

from pydantic import BaseModel
from pydantic.version import VERSION as pydantic_version

FILE_DATA_TYPE = "FILE"


class InferenceInput(BaseModel):
    name: str
    data: Any
    datatype: Literal["FILE", "FP32", "FP64", "INT32", "str"]
    content_type: Optional[str] = None
    shape: Union[int, List[int]]


class InferenceOutputField(BaseModel):
    name: str
    datatype: Literal["FILE", "FP32", "FP64", "INT32", "dict", "str"]
    content_type: Optional[str] = None
    data: Optional[str] = None


if pydantic_version >= '2':
    from pydantic import ConfigDict

    class InferenceArgs(BaseModel):
        model_key: str
        parameters: Optional[List[Tuple[Any, Any]]] = None
        inputs: List[InferenceInput]
        output_fields: List[InferenceOutputField]
        model: Optional[Any] = None

        model_config = ConfigDict(
            protected_namespaces=()
        )
else:
    class InferenceArgs(BaseModel):
        model_key: str
        parameters: Optional[List[Tuple[Any, Any]]] = None
        inputs: List[InferenceInput]
        output_fields: List[InferenceOutputField]
        model: Optional[Any] = None


class InferenceOutput(BaseModel):
    name: str
    data: Any
    datatype: Literal["FILE", "INT32", "FP32", "FP64", "dict", "str"]
    content_type: Optional[str] = None
    shape: Union[int, List[int]]


class InferenceReturns(BaseModel):
    outputs: List[InferenceOutput]


class InferenceProtocol(Protocol):

    def run_inference(self, model_key: str, inputs: List[InferenceInput],
                      output_fields: List[InferenceOutputField], parameters: List[Tuple[Any, Any]] = None,
                      model: Any = None) \
            -> List[InferenceOutput]:
        pass
