# ============================================================
# Система: Единая библиотека, Центр ИИ НИУ ВШЭ
# Модуль: MLComponent
# Авторы: Полежаев В.А., Хританков А.С.
# Дата создания: 2024 г.
# ============================================================
import logging
import posixpath
from typing import Optional

from pydantic import BaseModel

from unip.mlcmp.config import LICENSE_LOCAL_PATH, UNIP_MLCMP_APP_NAME, UNIP_MLCMP_PATH_PREFIX


class License(BaseModel):
    content: Optional[str] = None
    location: Optional[str] = None


def get_license() -> License:
    logger = logging.getLogger(__name__)
    if LICENSE_LOCAL_PATH:
        try:
            with open(LICENSE_LOCAL_PATH) as f:
                text = f.read()
                app_name = UNIP_MLCMP_APP_NAME.strip('/') if UNIP_MLCMP_APP_NAME else ''
                path_prefix = UNIP_MLCMP_PATH_PREFIX.strip('/') if UNIP_MLCMP_PATH_PREFIX else ''
                location = posixpath.join('/', app_name, path_prefix,
                                          'modelversion/{model_version}/license')
                return License(content=text, location=location)
        except Exception as exc:
            logger.exception('Cannot read local license file', exc_info=exc)
            return License()
    logger.warning('License not specified')
    return License()

